-- Sistemas Integrados TEKNISA
-- Apurao de ponto
--
-- Nome: 0000_TRGTEKO_FPA_GRAVA_PONTO.sql
--
-- Descrio:
-- Retorna as informaes de ponto do dia de um funcionario consolidadas e totalizadas
--
-- Parmetros:
-- P_NRORG - Nmero da organizao
-- P_CDEMPRESA - Cdigo da empresa
-- P_CDFUNC - Cdigo do funcionrio
-- P_DTPONTO - Data do ponto
-- P_OPERACAO - Indica o tipo de gravao para o ponto do dia:
--    'BATIDAPONTO' -> Limpa as tabelas FPA_PONTO e FPA_AUSENCIAPONTO e as gera novamente de acordo com os registros da BATIDAPONTO
--    'INCLUIPONTO' -> Incluso de registro de ponto na tabela FPA_PONTO
--    'ENTRADA' -> Alterao da data de Entrada da tabela FPA_PONTO
--    'SAIDA' -> Alterao da data de Sada da tabela FPA_PONTO
--    'AUSENCIAABONADA' -> Alterao manual (comandada pelo usurio) da quantidade de horas abonadas (intrajornada - justificadas)
--    'AUSENCIAEXTABONADA' -> Alterao manual (comandada pelo usurio) da quantidade de horas abonadas (extrajornada)
--    'AUSENCIANAOABON' -> Alterao manual (comandada pelo usurio) da quantidade de horas no abonadas (intrajornada - injustificadas)
--    'AUSENCIA' -> Incluso de perodo de ausncia abonado
--    'AUSENCIA_ALMOCO' -> Incluso de perodo de ausncia almoo
--    'ALMOCO' -> Atualiza ausncia como sendo perodo de almoo
--    'ABONO' -> Atualiza ausncia como sendo perodo abonado (justificado)
--    'NAOABONO' -> Atualiza ausncia como sendo perodo no abonado (injustificado)
--    'FALTA_COMPENSADA' -> Registra o ponto do dia como falta que ser compensada em banco de horas.
--    'FALTA_ABONADA' -> Registra o ponto do dia como falta abonada. Nesse caso, ser tb registrado qual o motivo da falta.
--    'AFASTAMENTO_INSS' -> Registra o ponto do dia como dia abonado por afastamento previsto pelo INSS. Nesse caso, ser tb registrado qual o motivo do afastamento.
--    'FERIAS' -> Registra o ponto do dia como dia abonado por gozo de frias. 
--    'FALTA_NAO_ABONADA' -> Registra o ponto do dia como falta no abonada. Nesse caso, alm do dia informado, o prximo domingo tb ser registrado como falta (perda de DSR).
--    Obs.: Exceto para a operao 'BATIDAPONTO', todas as outras recalculam as horas do dia (### ver varivel V_AJUSTAPONTO ###).
-- P_NRPONTO - (Opcional) Nmero sequencial (chave primria) que deve ser passado para alterao de dados nas operaes 'ENTRADA', 'SAIDA', 
--                       'AUSENCIAABONADA', 'AUSENCIAEXTABONADA', 'AUSENCIANAOABON', 'ALMOCO', 'ABONO', 'NAOABONO', 'FALTA_ABONADA' e 'AFASTAMENTO_INSS'.
-- P_DTINICIO - (Opcional) Data de incio de perodo. Deve ser passada para as operaes 'INCLUIPONTO', 'ENTRADA', 'AUSENCIA' e 'AUSENCIA_ALMOCO'.
-- P_DTFIM - (Opcional) Data de incio de perodo. Deve ser passada para as operaes 'INCLUIPONTO', 'SAIDA', 'AUSENCIA' e 'AUSENCIA_ALMOCO'.
-- P_HROPERACAO - (Opcional) Qtde de horas referente s operaes 'AUSENCIAABONADA', 'AUSENCIAEXTABONADA', 'AUSENCIANAOABON'.
-- P_DSOBSERVACAO - (Opcional) Observao a respeito da operao.  obrigatria nas operaes 'ENTRADA', 'SAIDA', 'AUSENCIAABONADA', 
--                            'AUSENCIAEXTABONADA', 'AUSENCIANAOABON', 'AUSENCIA' e 'ABONO'.
-- Criao:
-- 21/09/2012 Newton Neves dos Santos

CREATE OR REPLACE 
PROCEDURE FPA_GRAVA_PONTO_DIA
  (P_NRORG IN FPA_PONTO.NRORG%TYPE,
   P_CDEMPRESA IN EMPRESA.CDEMPRESA%TYPE,
   P_CDFUNC IN FUNCIONARIO.CDFUNC%TYPE,
   P_DTPONTO IN BATIDAPONTO.DTBATIDAPTO%TYPE,
   P_OPERACAO IN VARCHAR2,
   P_NRPONTO IN FPA_PONTO.NRPONTO%TYPE DEFAULT NULL,
   P_DTINICIO IN FPA_AUSENCIAPONTO.DTINICIOAUSENCIA%TYPE DEFAULT NULL,
   P_DTFIM IN FPA_AUSENCIAPONTO.DTFIMAUSENCIA%TYPE DEFAULT NULL,
   P_HROPERACAO IN FPA_PONTO.HRREMUNERADA%TYPE DEFAULT NULL,
   P_DSOBSERVACAO IN FPA_AUSENCIAPONTO.DSOBSERVACAOAUSENCIA%TYPE DEFAULT NULL) 

IS

   V_DTAUX FPA_AUSENCIAPONTO.DTFIMAUSENCIA%TYPE;
   V_DTHRENTRADA_ANT FPA_AUSENCIAPONTO.DTFIMAUSENCIA%TYPE;
   
   V_ST_NRPONTO NOVOCODIGO.NRSEQUENCIAL%TYPE;
   V_NRPONTO NUMBER;
   V_IDPONTO FPA_PONTO.IDPONTO%TYPE;
   V_DTPONTO FPA_PONTO.DTPONTO%TYPE;
   V_DIASEMPTO VARCHAR2(01);
   V_DTINISEM FPA_PONTO.DTPONTO%TYPE;
   V_DTFIMSEM FPA_PONTO.DTPONTO%TYPE;
   V_ST_AUSENCIAPONTO NOVOCODIGO.NRSEQUENCIAL%TYPE;
   V_AUSENCIAPONTO NUMBER;
   V_COUNT_AP NUMBER;
   V_IDPROXPONTO NUMBER;
   
   V_ERROR_MESSAGE VARCHAR2(255);
   V_ERROR_CODE NUMBER;
   
   V_AJUSTAPONTO BOOLEAN;
   V_EXISTE_CDA BOOLEAN;
   V_FALTA_NA_SEMANA BOOLEAN;
   V_SEM_BATIDA_PONTO BOOLEAN;
   
   V_HRENTRADA VARCHAR2(4);
   V_HRSAIDA VARCHAR2(4);
   V_HRTEMPOCALCULADO VARCHAR2(04);
   V_HRTEMPOCALCULADO_5 VARCHAR2(05);
   V_IDTEMPOABONADO FPA_AUSENCIAPONTO.IDAUSENCIAABONADA%TYPE;
   V_IDTEMPONAOABON FPA_AUSENCIAPONTO.IDAUSENCIANAOABON%TYPE;
   V_IDALMOCO FPA_AUSENCIAPONTO.IDAUSENCIAALMOCO%TYPE;

   V_CDTPADMISSAO FUNCIONARIO.CDTPADMISSAO%TYPE;
   V_SGESTADO_FUNC ENDEFILI.SGESTADO%TYPE;
   V_CDEMPRESAFOLHA FUNCIONARIO.CDEMPRESAFOLHA%TYPE;
   V_HBENTRADA VARCHAR2(4);
   V_HBSAIDAALMOCO VARCHAR2(4);
   V_HBVOLTAALMOCO VARCHAR2(4);
   V_HBSAIDA VARCHAR2(4);
   V_HORASJORNADA VARCHAR2(04);
   V_HRREMUNERADA VARCHAR2(04);  
   V_TOT_HRREMUNERADA VARCHAR2(05);  
   V_TOT_HRSALDO VARCHAR2(06);  
   V_SAB_DOM_FERIADO VARCHAR2(01);
 
   V_NRAUSENCIAPONTO FPA_AUSENCIAPONTO.NRAUSENCIAPONTO%TYPE;
   V_DTHRSAIDA FPA_AUSENCIAPONTO.DTINICIOAUSENCIA%TYPE;
   V_DTHRENTRADA FPA_AUSENCIAPONTO.DTFIMAUSENCIA%TYPE;

   V_IDOCORRENCIA VARCHAR2(20);
   V_DTSAIDA FPA_AUSENCIAPONTO.DTINICIOAUSENCIA%TYPE;
   V_DTENTRADA FPA_AUSENCIAPONTO.DTFIMAUSENCIA%TYPE;
   V_AUSENCIATOTAL VARCHAR2(04);
   V_AUSENCIAABONADAORIG VARCHAR2(04);
   V_AUSENCIAABONADA VARCHAR2(04);
   V_AUSENCIAEXTABONORIG VARCHAR2(04);
   V_AUSENCIAEXTABON VARCHAR2(04);
   V_AUSENCIANAOABONORIG VARCHAR2(04);
   V_AUSENCIANAOABON VARCHAR2(04);
   V_AUSENCIAALMOCO VARCHAR2(04);
   V_HORASREMUNERADAS VARCHAR2(04);
   V_QTHORASBASE VARCHAR2(04);
   V_ALMOCOHORABASE VARCHAR2(04);
   V_QT_ALMOCO_HB VARCHAR2(09);
   V_QTHB_TOT VARCHAR2(04);
   V_ALMOCOHB_TOT VARCHAR2(04);
   V_DTINICIOAUSENCIA FPA_AUSENCIAPONTO.DTINICIOAUSENCIA%TYPE;
   V_DTFIMAUSENCIA FPA_AUSENCIAPONTO.DTFIMAUSENCIA%TYPE;
   V_DTINICIOAUX FPA_AUSENCIAPONTO.DTINICIOAUSENCIA%TYPE;
   V_DTFIMAUX FPA_AUSENCIAPONTO.DTFIMAUSENCIA%TYPE;
   V_ALMOCOAUSENCIA VARCHAR2(04);
   V_DEBITOCREDITOHORAS VARCHAR2(05);
   V_DEBCREDBANCOHORAS VARCHAR2(05);
   V_DEBITOCREDITOHORASEXTRA VARCHAR2(05);
   V_IDFALTA VARCHAR2(01);
   V_IDAUTDEBBH VARCHAR2(01);
   
   V_CDCLIENTE SERVPREST.CDCLIENTE%TYPE;
   V_CDPROSPECT SERVPREST.CDPROSPECT%TYPE;
   V_NRSERVPREST SERVPREST.NRSERVPREST%TYPE;
   V_QTHRTOTAL SERVPREST.QTHRTOTAL%TYPE;
   
   P_DTPERIODO_ANTE FPA_PERIODOAPURAPONTO.DTINICIALAPURAPONTO%TYPE;
   V_DTINICIALAPURAPONTO FPA_PERIODOAPURAPONTO.DTINICIALAPURAPONTO%TYPE;
   V_DTFINALAPURAPONTO FPA_PERIODOAPURAPONTO.DTFINALAPURAPONTO%TYPE;
   V_NRPERIODOAPURAPONTO FPA_PERIODOAPURAPONTO.NRPERIODOAPURAPONTO%TYPE;
   V_NRPERIODOAPURA_ANTE FPA_PERIODOAPURAPONTO.NRPERIODOAPURAPONTO%TYPE;
  
   V_HRAUX FPA_TOTALIZAPONTO.HRSALDOBANCOHORAS%TYPE;
   V_HRSALDOBH_MES FPA_TOTALIZAPONTO.HRSALDOBANCOHORAS%TYPE;
   V_HRCOMPBH FPA_TOTALIZAPONTO.HRCOMPBANCOHORAS%TYPE;
   V_HRFALTA FPA_TOTALIZAPONTO.HRSALDOHORASFALTA%TYPE;
   V_HREXTRA25BH FPA_TOTALIZAPONTO.HREXTRA25BANCOHORAS%TYPE;
   V_HREXTRA25BH_ACRESC FPA_TOTALIZAPONTO.HREXTRA25BANCOHORAS%TYPE;
   V_HREXTRA50BH FPA_TOTALIZAPONTO.HREXTRA50BANCOHORAS%TYPE;
   V_HREXTRA50BH_ACRESC FPA_TOTALIZAPONTO.HREXTRA50BANCOHORAS%TYPE;
   V_HREXTRA100BH FPA_TOTALIZAPONTO.HREXTRA100BANCOHORAS%TYPE;
   V_HREXTRA100BH_ACRESC FPA_TOTALIZAPONTO.HREXTRA100BANCOHORAS%TYPE;

   V_HRCOMPBANCOHORAS FPA_TOTALIZAPONTO.HRCOMPBANCOHORAS%TYPE;
   V_HRSALDOHORASFALTA FPA_TOTALIZAPONTO.HRSALDOHORASFALTA%TYPE;
   V_HRSALDOBANCOHORAS FPA_TOTALIZAPONTO.HRSALDOBANCOHORAS%TYPE;
   V_HRSALDOBHRSACRESC FPA_TOTALIZAPONTO.HRSALDOBHRSACRESC%TYPE;
   V_IDACERTOBANCOHORAS FPA_TOTALIZAPONTO.IDACERTOBANCOHORAS%TYPE;
	V_IDACERTOBH_ANT FPA_TOTALIZAPONTO.IDACERTOBANCOHORAS%TYPE;
   V_HREXTRA25BANCOHORAS FPA_TOTALIZAPONTO.HREXTRA25BANCOHORAS%TYPE;
   V_HREXTRA50BANCOHORAS FPA_TOTALIZAPONTO.HREXTRA50BANCOHORAS%TYPE;
   V_HREXTRA100BANCOHORAS FPA_TOTALIZAPONTO.HREXTRA100BANCOHORAS%TYPE;
   V_HREXTRA100 FPA_TOTALIZAPONTO.HREXTRA100%TYPE;
   V_QTDIASDESCONTOFALTA FPA_TOTALIZAPONTO.QTDIASDESCONTOFALTA%TYPE;
   V_QTDIASDESCONTODSR FPA_TOTALIZAPONTO.QTDIASDESCONTODSR%TYPE;
   
   CURSOR C_FPAAUSENCIA IS 
       SELECT NRAUSENCIAPONTO, DTINICIOAUSENCIA, DTFIMAUSENCIA, HRAUSENCIA, IDAUSENCIAABONADA, IDAUSENCIANAOABON, IDAUSENCIAALMOCO
         FROM FPA_AUSENCIAPONTO
        WHERE NRORG = P_NRORG
          AND CDEMPRESA = P_CDEMPRESA
          AND CDFUNC = P_CDFUNC
          AND DTPONTO = P_DTPONTO
        ORDER BY DTINICIOAUSENCIA;

   CURSOR C_FPA_PONTO_DIA IS 
       SELECT F.IDOCORRENCIA,
              DECODE(F.IDOCORRENCIA, 'ENTRADASAIDA', TO_DATE(TO_CHAR(F.DTHRENTRADA, 'DD/MM/YYYY')||F.HRENTRADA, 'DD/MM/YYYYHH24:MI'), NULL) AS DTENTRADA,
              DECODE(F.IDOCORRENCIA, 'ENTRADASAIDA', TO_DATE(TO_CHAR(F.DTHRSAIDA, 'DD/MM/YYYY')||F.HRSAIDA, 'DD/MM/YYYYHH24:MI'), NULL) AS DTSAIDA, 
              SUBSTR(F.HRTEMPOCALCULADO,1,4) AS HRTEMPOCALCULADO,
              F.HRTEMPOCALCULADO AS HRTEMPOCALCULADO_5
         FROM TABLE (FPA_PONTO_DIA (P_CDEMPRESA, P_CDFUNC, P_DTPONTO, 'BATIDAPONTO')) F
        WHERE IDOCORRENCIA IN ('ENTRADASAIDA', 'AUSENCIAABONADA', 'AUSENCIANAOABONADA', 'AUSENCIATOTAL', 'HORASREMUNERADAS', 'DEBITOCREDITOHORAS', 'DEBCREDHORASEXTRA');

   CURSOR C_FPA_CDA_DIA IS 
       SELECT CDCLIENTE, CDPROSPECT, NRSERVPREST, NVL(QTHRTOTAL, '0000') AS QTHRTOTAL
         FROM SERVPREST
        WHERE NRORG = P_NRORG
          AND CDEMPRESA = P_CDEMPRESA
          AND CDFUNC = P_CDFUNC
          AND DTSERVPREST = P_DTPONTO;
          
   CURSOR C_FPA_PONTO_TOTAL IS 
       SELECT DT.DTDATA, DECODE(FF.DTFERIFILI, NULL, 'N', 'S'),
              TO_CHAR(DT.DTDATA, 'D'), PT.NRPONTO, NVL(PT.IDPONTO, 'ATIVIDADE_INTERNA'), NVL(PT.HRSALDO, ' 0000'), NVL(PT.HRSALDOHORAEXTRA, ' 0000'), 
              PT.HRAUSENCIAALMOCO, UTILIT.SUBT_HORA2(TO_CHAR(PT.DTSAIDA, 'HH24MI'), TO_CHAR(PT.DTENTRADA, 'HH24MI')), NVL(PT.HRREMUNERADA, '0000'), 
              PT.IDFALTA, FPA_ALMOCO_HORABASE(DT.CDEMPRESA, DT.CDFUNC, DT.DTDATA, DT.NRORG, '1') AS ALMOCOHORABASE, IDAUTDEBBANCOHORAS
         FROM FPA_PONTO PT, FERIFILI FF,
              (SELECT FU.NRORG, FU.CDEMPRESA, FU.CDFUNC, FU.CDFILIALOCFUNC, DATA.DTDATA
                 FROM DATA, FUNCIONARIO FU
                WHERE FU.NRORG = P_NRORG
                  AND FU.CDEMPRESA = P_CDEMPRESA
                  AND FU.CDFUNC = P_CDFUNC
                  AND DATA.DTDATA BETWEEN V_DTINICIOAUX AND V_DTFIMAUX) DT
        WHERE DT.NRORG = PT.NRORG(+)
          AND DT.CDEMPRESA = PT.CDEMPRESA(+)
          AND DT.CDFUNC = PT.CDFUNC(+)
          AND DT.DTDATA = PT.DTPONTO(+)
          AND DT.NRORG = FF.NRORG(+) 
          AND DT.CDFILIALOCFUNC = FF.CDFILIAL(+) 
          AND DT.DTDATA = FF.DTFERIFILI(+)
        ORDER BY DT.DTDATA;

BEGIN
   V_AJUSTAPONTO := TRUE;
   
   /* Verificao de dia til/no til */
   BEGIN
      SELECT DECODE(FF.DTFERIFILI, NULL, 'N', 'S')
        INTO V_SAB_DOM_FERIADO
        FROM FERIFILI FF, FUNCIONARIO FU
       WHERE FU.NRORG = P_NRORG 
         AND FU.CDEMPRESA = P_CDEMPRESA 
         AND FU.CDFUNC = P_CDFUNC
         AND FF.NRORG = FU.NRORG
         AND FF.CDFILIAL = FU.CDFILIALOCFUNC
         AND FF.DTFERIFILI = P_DTPONTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_SAB_DOM_FERIADO := 'N';
      END;
   END;

   /* Verificao dos tempos do horrio base (TEMPO TOTAL DIRIO e TEMPO ALMOO) e do tipo de admisso do funcionrio */
   BEGIN
      SELECT FUN.CDTPADMISSAO, 
             EFI.SGESTADO, 
             FUN.CDEMPRESAFOLHA, 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT1HBFUN, HBE.HRENT1HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRSAI1HBFUN, HBE.HRSAI1HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRENT2HBFUN, HBE.HRENT2HBEXC), 'HH24MI'), 
             TO_CHAR(DECODE(HBE.DTHORABASEEXC, NULL, HBF.HRSAI2HBFUN, HBE.HRSAI2HBEXC), 'HH24MI')
        INTO V_CDTPADMISSAO, 
             V_SGESTADO_FUNC, 
             V_CDEMPRESAFOLHA, 
             V_HBENTRADA, 
             V_HBSAIDAALMOCO, 
             V_HBVOLTAALMOCO, 
             V_HBSAIDA
        FROM FUNCIONARIO FUN, FPA_HORABASEEXC HBE, ENDEFILI EFI,
             (SELECT CDEMPRESA, CDFUNC, HRENT1HBFUN, HRSAI1HBFUN, HRENT2HBFUN, HRSAI2HBFUN
                FROM HORBASFU
               WHERE CDFUNC = P_CDFUNC
                 AND CDEMPRESA = P_CDEMPRESA
                 AND IDDIHBFUN = TO_CHAR(P_DTPONTO, 'D')
                 AND DTINICHBFUN = (SELECT MAX(DTINICHBFUN)
                                      FROM HORBASFU
                                     WHERE CDFUNC = P_CDFUNC
                                       AND CDEMPRESA = P_CDEMPRESA
                                       AND DTINICHBFUN <= P_DTPONTO)) HBF
       WHERE FUN.CDFUNC = P_CDFUNC
         AND FUN.CDEMPRESA = P_CDEMPRESA
         AND FUN.CDFILIALOCFUNC = EFI.CDFILIAL(+)
         AND EFI.IDTPENDEFILI = 'P'
         AND FUN.CDFUNC = HBF.CDFUNC(+)
         AND FUN.CDEMPRESA = HBF.CDEMPRESA(+)
         AND FUN.CDTPADMISSAO = HBE.CDTPADMISSAO(+)
         AND P_DTPONTO = HBE.DTHORABASEEXC(+);          
         
      IF (V_HBENTRADA IS NULL) OR (V_SAB_DOM_FERIADO = 'S') THEN
         V_QTHORASBASE := '0000';
         V_ALMOCOHORABASE := '0000';
      ELSE
         IF V_HBVOLTAALMOCO IS NULL THEN
            V_QTHORASBASE := UTILIT.SUBT_HORA2(V_HBSAIDAALMOCO, V_HBENTRADA);
            V_ALMOCOHORABASE := '0000';
         ELSE
            V_QTHORASBASE := UTILIT.SOMA_HORA(UTILIT.SUBT_HORA2(V_HBSAIDAALMOCO, V_HBENTRADA), UTILIT.SUBT_HORA2(V_HBSAIDA, V_HBVOLTAALMOCO));
            V_ALMOCOHORABASE := UTILIT.SUBT_HORA2(V_HBVOLTAALMOCO, V_HBSAIDAALMOCO);
         END IF;   
      END IF;      
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_QTHORASBASE := '0000';
         V_ALMOCOHORABASE := '0000';
      END;
   END;
   
   /* Verificao do perodo de apurao de acordo com a tabela FPA_PERIODOAPURAPONTO */
   SELECT P.NRPERIODOAPURAPONTO, P.DTINICIALAPURAPONTO, P.DTFINALAPURAPONTO
     INTO V_NRPERIODOAPURAPONTO, V_DTINICIALAPURAPONTO, V_DTFINALAPURAPONTO
     FROM FPA_PERIODOAPURAPONTO P, FUNCIONARIO F
    WHERE F.CDEMPRESA = P_CDEMPRESA
      AND F.CDFUNC = P_CDFUNC
      AND F.NRORG = P_NRORG
      AND P.CDTPADMISSAO = V_CDTPADMISSAO
      AND P.DTINICIALAPURAPONTO <= P_DTPONTO
      AND P.DTFINALAPURAPONTO >= P_DTPONTO;

   /* Tratamento de excluso de registros das tabelas FPA_PONTO e FPA_AUSENCIAPONTO 
      antes das operaes de insero de dados nessas tabelas */
   IF P_OPERACAO IN ('BATIDAPONTO', 'INCLUIPONTO', 'FALTA_COMPENSADA', 'FALTA_ABONADA', 'FALTA_NAO_ABONADA') THEN
      UPDATE SERVPREST
         SET NRPONTO = NULL
       WHERE NRORG = P_NRORG
         AND NRPONTO IN (SELECT NRPONTO
                           FROM FPA_PONTO
                          WHERE NRORG = P_NRORG
                            AND CDEMPRESA = P_CDEMPRESA
                            AND CDFUNC = P_CDFUNC
                            AND DTPONTO = P_DTPONTO);
                            
      BEGIN
         SELECT IDFALTA
           INTO V_IDFALTA
           FROM FPA_PONTO
          WHERE NRORG = P_NRORG
            AND CDEMPRESA = P_CDEMPRESA
            AND CDFUNC = P_CDFUNC
            AND DTPONTO = P_DTPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_IDFALTA := 'N';
         END;
      END;       
                            
      DELETE FROM FPA_PONTO
       WHERE NRORG = P_NRORG
         AND CDEMPRESA = P_CDEMPRESA
         AND CDFUNC = P_CDFUNC
         AND DTPONTO = P_DTPONTO;
      
      DELETE FROM FPA_AUSENCIAPONTO
       WHERE NRORG = P_NRORG
         AND CDEMPRESA = P_CDEMPRESA
         AND CDFUNC = P_CDFUNC
         AND DTPONTO = P_DTPONTO;
   END IF;
         
   IF P_OPERACAO = 'BATIDAPONTO' THEN
      BEGIN
         /* Comando para verificao de intervalo de dias entre a data do ltimo lanamento e a data que est sendo lanada.     */
         /* Se V_IDPROXPONTO for maior que 1 (um), significa que o usurio est tentando lanar o ponto de forma no sequencial. */
         /* Essa consistncia no ser realizada para colaboradores com tipo de admissao 12x36 (CDTIPOADMISSAO = 'D').           */
         IF V_CDTPADMISSAO <> 'D' THEN
            SELECT COUNT(*)
              INTO V_IDPROXPONTO
              FROM DATA DT, (SELECT MAX(PT.DTPONTO) AS DTMAXPONTO
                              FROM FPA_PONTO PT
                             WHERE PT.NRORG = P_NRORG
                               AND PT.CDEMPRESA = P_CDEMPRESA
                               AND PT.CDFUNC = P_CDFUNC
                               AND PT.DTPONTO BETWEEN V_DTINICIALAPURAPONTO AND V_DTFINALAPURAPONTO) MP
             WHERE DT.DTDATA BETWEEN MP.DTMAXPONTO+1 AND P_DTPONTO
               AND DT.DTDATA NOT IN (SELECT FF.DTFERIFILI
                                       FROM FERIFILI FF, FUNCIONARIO FU
                                      WHERE FU.CDEMPRESA = P_CDEMPRESA
                                        AND FU.CDFUNC = P_CDFUNC
                                        AND FF.CDFILIAL = FU.CDFILIALOCFUNC
                                        AND FF.DTFERIFILI = DT.DTDATA);
            
            IF V_IDPROXPONTO > 1 THEN 
               RAISE_APPLICATION_ERROR (-20000, 'ERROR_PONTO_SEQUENCIAL');    
            END IF;
         END IF;   
      
         NOVO_CODIGO('FPA_PONTO',V_ST_NRPONTO, 1, P_NRORG);
         V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
   
         OPEN C_FPA_PONTO_DIA;
         LOOP
            FETCH C_FPA_PONTO_DIA INTO V_IDOCORRENCIA, V_DTHRENTRADA, V_DTHRSAIDA, V_HRTEMPOCALCULADO, V_HRTEMPOCALCULADO_5;
            EXIT WHEN C_FPA_PONTO_DIA%NOTFOUND;   

            IF V_IDOCORRENCIA = 'ENTRADASAIDA' THEN
               V_DTENTRADA := V_DTHRENTRADA;
               V_DTSAIDA := V_DTHRSAIDA;
            END IF;
            IF V_IDOCORRENCIA = 'AUSENCIAABONADA' THEN
               V_AUSENCIAABONADAORIG := V_HRTEMPOCALCULADO;
               V_AUSENCIAABONADA := V_HRTEMPOCALCULADO;
            END IF;
            IF V_IDOCORRENCIA = 'AUSENCIANAOABONADA' THEN
               V_AUSENCIANAOABONORIG := V_HRTEMPOCALCULADO;
               V_AUSENCIANAOABON := V_HRTEMPOCALCULADO;
            END IF;
            IF V_IDOCORRENCIA = 'AUSENCIATOTAL' THEN
               V_AUSENCIATOTAL := V_HRTEMPOCALCULADO;
            END IF;
            IF V_IDOCORRENCIA = 'HORASREMUNERADAS' THEN
               V_HORASREMUNERADAS := V_HRTEMPOCALCULADO;
            END IF;
            IF V_IDOCORRENCIA = 'DEBITOCREDITOHORAS' THEN
               V_DEBITOCREDITOHORAS := V_HRTEMPOCALCULADO_5;
            END IF;
            IF V_IDOCORRENCIA = 'DEBCREDHORASEXTRA' THEN
               V_DEBITOCREDITOHORASEXTRA := V_HRTEMPOCALCULADO_5;
            END IF;
            
         END LOOP;
         CLOSE C_FPA_PONTO_DIA;

         IF NOT (V_DTENTRADA IS NULL) THEN
            INSERT INTO FPA_PONTO (NRPONTO, 
                                   IDPONTO,
                                   CDEMPRESA, 
                                   CDFUNC, 
                                   DTPONTO, 
                                   DTENTRADAORIG, 
                                   DTENTRADA, 
                                   DTSAIDAORIG, 
                                   DTSAIDA,
                                   HRAUSENCIATOTAL, 
                                   HRAUSENCIAALMOCO,
                                   HRAUSENCIAABONADAORIG, 
                                   HRAUSENCIAABONADA, 
                                   HRAUSENCIAEXTABONORIG,
                                   HRAUSENCIAEXTABON,
                                   HRAUSENCIANAOABONORIG,
                                   HRAUSENCIANAOABON,
                                   HRREMUNERADA, 
                                   HRSALDO,
                                   HRSALDOHORAEXTRA,
                                   IDFALTA)
                 VALUES (V_NRPONTO,
                         'ATIVIDADE_INTERNA', 
                         P_CDEMPRESA,
                         P_CDFUNC,
                         P_DTPONTO,
                         V_DTENTRADA,
                         V_DTENTRADA,
                         V_DTSAIDA,
                         V_DTSAIDA,
                         V_AUSENCIATOTAL,
                         '0000',
                         V_AUSENCIAABONADAORIG,
                         V_AUSENCIAABONADA,
                         '0000',
                         '0000',
                         V_AUSENCIANAOABONORIG,
                         V_AUSENCIANAOABON,
                         V_HORASREMUNERADAS,
                         V_DEBITOCREDITOHORAS,
                         V_DEBITOCREDITOHORASEXTRA,
                         V_IDFALTA);
         
            NOVO_CODIGO_SC('FPA_AUSENCIAPONTO',V_ST_AUSENCIAPONTO, 1, P_NRORG);
            V_AUSENCIAPONTO := TO_NUMBER(V_ST_AUSENCIAPONTO)-1;

            INSERT INTO FPA_AUSENCIAPONTO (NRAUSENCIAPONTO, 
                                           CDEMPRESA, 
                                           CDFUNC, 
                                           DTPONTO, 
                                           DTINICIOAUSENCIA, 
                                           DTFIMAUSENCIA, 
                                           HRAUSENCIA, 
                                           IDAUSENCIAABONADA, 
                                           IDAUSENCIANAOABON, 
                                           IDAUSENCIAALMOCO, 
                                           DSOBSERVACAOAUSENCIA)
            SELECT ROWNUM+V_AUSENCIAPONTO AS NRAUSENCIAPONTO, 
                   F.CDEMPRESA AS CDEMPRESA, 
                   F.CDFUNC AS CDFUNC, 
                   P_DTPONTO AS DTPONTO, 
                   DTHRSAIDA AS DTINICIOAUSENCIA, 
                   DTHRENTRADA AS DTFIMAUSENCIA, 
                   HRTEMPOCALCULADO AS HRAUSENCIA, 
                   IDTEMPOABONADO AS IDAUSENCIAABONADA, 
                   IDTEMPONAOABON AS IDAUSENCIANAOABON,
                   IDALMOCO AS IDAUSENCIAALMOCO, 
                   DSOBSERVACAO AS DSOBSERVACAOAUSENCIA
              FROM TABLE (FPA_PONTO_DIA (P_CDEMPRESA, P_CDFUNC, P_DTPONTO, 'BATIDAPONTO')) F
             WHERE F.IDOCORRENCIA = 'AUSENCIA';
             
            SELECT COUNT(*) 
              INTO V_COUNT_AP
              FROM FPA_AUSENCIAPONTO
             WHERE NRORG = P_NRORG
               AND CDEMPRESA = P_CDEMPRESA
               AND CDFUNC = P_CDFUNC
               AND DTPONTO = P_DTPONTO;

            IF (NOT V_COUNT_AP IS NULL) AND (V_COUNT_AP <> 0) THEN
               NOVO_CODIGO_SC('FPA_AUSENCIAPONTO',V_ST_AUSENCIAPONTO, V_COUNT_AP-1, P_NRORG);
            END IF;
         ELSE
            V_EXISTE_CDA := FALSE;
            V_HORASREMUNERADAS := '0000';
         
            OPEN C_FPA_CDA_DIA;
            LOOP
               FETCH C_FPA_CDA_DIA INTO V_CDCLIENTE, V_CDPROSPECT, V_NRSERVPREST, V_QTHRTOTAL;
               EXIT WHEN C_FPA_CDA_DIA%NOTFOUND;   
               
               V_HORASREMUNERADAS := UTILIT.SOMA_HORA(V_HORASREMUNERADAS, V_QTHRTOTAL);
               
               V_EXISTE_CDA := TRUE;
            END LOOP;
            CLOSE C_FPA_CDA_DIA;
            
            IF V_EXISTE_CDA THEN
               IF (V_HORASREMUNERADAS <= V_QTHORASBASE) OR (V_CDTPADMISSAO IN ('S', 'P')) THEN
                  V_HORASREMUNERADAS := V_QTHORASBASE;
                  V_DEBITOCREDITOHORAS := ' 0000';
                  V_DEBITOCREDITOHORASEXTRA := ' 0000';
               ELSE
                  V_DEBITOCREDITOHORAS := UTILIT.SUBT_HORA2(V_HORASREMUNERADAS, V_QTHORASBASE);  
                  
                  /* Tratamento para Scios e PJ (V_QTHORASBASE <> '0000') */
                  
                  /* #### Clculo atual considerando banco de horas #### */
                  V_DEBITOCREDITOHORASEXTRA := ' 0000';
                  
                  /* #### Clculo considerando horas extras a partir de 02:00 horas positivas (funcionrios alocados em MG) 
                   * ou todo saldo positivo como hora extra (funcionrios alocados em outros estados) 
						 * ou todo saldo positivo como hora extra (Sb, Dom e Feriados - Solicitado por Gabrielle em 13/11/2014) #### */
                  IF NOT (V_CDTPADMISSAO IN ('B', 'E', 'O')) THEN                
                     IF V_SGESTADO_FUNC = 'MG' THEN
                        IF (NOT (V_CDTPADMISSAO IN ('S', 'P'))) THEN
                           IF (V_SAB_DOM_FERIADO = 'N') THEN 
										IF (V_DEBITOCREDITOHORAS > '0200') THEN
                                 V_DEBITOCREDITOHORASEXTRA := UTILIT.SUBT_HORA2(V_DEBITOCREDITOHORAS, '0200');
										END IF;	
									ELSE	
										V_DEBITOCREDITOHORASEXTRA := V_DEBITOCREDITOHORAS;
                           END IF;                     
                        END IF;   
                     ELSE
                        V_DEBITOCREDITOHORASEXTRA := V_DEBITOCREDITOHORAS;
                     END IF;                     
                  END IF;   
               END IF;
            
               INSERT INTO FPA_PONTO (NRPONTO,
                                      IDPONTO,
                                      CDEMPRESA, 
                                      CDFUNC, 
                                      DTPONTO, 
                                      DTENTRADAORIG, 
                                      DTENTRADA, 
                                      DTSAIDAORIG, 
                                      DTSAIDA,
                                      HRAUSENCIATOTAL, 
                                      HRAUSENCIAALMOCO,
                                      HRAUSENCIAABONADAORIG, 
                                      HRAUSENCIAABONADA, 
                                      HRAUSENCIAEXTABONORIG,
                                      HRAUSENCIAEXTABON,
                                      HRAUSENCIANAOABONORIG,
                                      HRAUSENCIANAOABON,
                                      HRREMUNERADA, 
                                      HRSALDO,
                                      HRSALDOHORAEXTRA,
                                      IDFALTA)
                    VALUES (V_NRPONTO,
                            'ATIVIDADE_EXTERNA',
                            P_CDEMPRESA,
                            P_CDFUNC,
                            P_DTPONTO,
                            P_DTPONTO,
                            P_DTPONTO,
                            P_DTPONTO,
                            P_DTPONTO,
                            NULL,
                            NULL,
                            NULL,
                            NULL,
                            NULL,
                            NULL,
                            NULL,
                            NULL,
                            V_HORASREMUNERADAS,
                            V_DEBITOCREDITOHORAS,
                            V_DEBITOCREDITOHORASEXTRA,
                            V_IDFALTA);            
                              
               UPDATE SERVPREST
                  SET NRORG = P_NRORG,
                      NRPONTO = V_NRPONTO
                WHERE NRORG = P_NRORG
                  AND CDEMPRESA = P_CDEMPRESA
                  AND CDFUNC = P_CDFUNC
                  AND DTSERVPREST = P_DTPONTO;
                  
               V_SEM_BATIDA_PONTO := FALSE;   
            ELSE
               V_SEM_BATIDA_PONTO := TRUE;
            END IF;   
         END IF;   
         
         V_AJUSTAPONTO := FALSE;
         
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20000, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;   
   
   IF P_OPERACAO = 'FALTA_COMPENSADA' THEN
      BEGIN
         NOVO_CODIGO('FPA_PONTO',V_ST_NRPONTO, 1, P_NRORG);
         V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
         
         IF V_QTHORASBASE <> '0000' THEN
            V_HRTEMPOCALCULADO_5 := '-'||V_QTHORASBASE;
         ELSE
            V_HRTEMPOCALCULADO_5 := V_QTHORASBASE;
         END IF;         
         
         IF V_CDTPADMISSAO <> 'E' THEN
            V_IDAUTDEBBH := 'S';
         ELSE
            V_IDAUTDEBBH := 'N';
         END IF;         

         INSERT 
            INTO FPA_PONTO (NRORG, NRPONTO, IDPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRREMUNERADA, HRSALDO, IDAUTDEBBANCOHORAS)
          VALUES (P_NRORG, V_NRPONTO, 'FALTA_COMPENSADA', P_CDEMPRESA, P_CDFUNC, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, '0000', V_HRTEMPOCALCULADO_5, V_IDAUTDEBBH);

         V_AJUSTAPONTO := FALSE;

      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20000, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;
   
   IF P_OPERACAO = 'FALTA_ABONADA' THEN
      BEGIN
         NOVO_CODIGO('FPA_PONTO',V_ST_NRPONTO, 1, P_NRORG);
         V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
         
         INSERT 
           INTO FPA_PONTO (NRORG, NRPONTO, IDPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRREMUNERADA, HRSALDO, NRMOTIVOEXCEFREQ)
         VALUES (P_NRORG, V_NRPONTO, 'FALTA_ABONADA', P_CDEMPRESA, P_CDFUNC, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, V_QTHORASBASE, ' 0000', P_NRPONTO);

         V_AJUSTAPONTO := FALSE;

      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20000, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;
   
   IF P_OPERACAO = 'AFASTAMENTO_INSS' THEN
      BEGIN
         NOVO_CODIGO('FPA_PONTO',V_ST_NRPONTO, 1, P_NRORG);
         V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
         
         INSERT 
           INTO FPA_PONTO (NRORG, NRPONTO, IDPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRREMUNERADA, HRSALDO, NRMOTIVOEXCEFREQ)
         VALUES (P_NRORG, V_NRPONTO, 'AFASTAMENTO_INSS', P_CDEMPRESA, P_CDFUNC, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, V_QTHORASBASE, ' 0000', P_NRPONTO);

         V_AJUSTAPONTO := FALSE;

      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20000, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;
   
   IF P_OPERACAO = 'FERIAS' THEN
      BEGIN
         NOVO_CODIGO('FPA_PONTO',V_ST_NRPONTO, 1, P_NRORG);
         V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
         
         INSERT 
           INTO FPA_PONTO (NRORG, NRPONTO, IDPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRREMUNERADA, HRSALDO)
         VALUES (P_NRORG, V_NRPONTO, 'FERIAS', P_CDEMPRESA, P_CDFUNC, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, V_QTHORASBASE, ' 0000');

         V_AJUSTAPONTO := FALSE;

      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20000, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;
   
   IF P_OPERACAO = 'FALTA_NAO_ABONADA' THEN
      BEGIN
         NOVO_CODIGO('FPA_PONTO',V_ST_NRPONTO, 1, P_NRORG);
         V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
         
         INSERT 
           INTO FPA_PONTO (NRORG, NRPONTO, IDPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRREMUNERADA, HRSALDO, IDFALTA)
         VALUES (P_NRORG, V_NRPONTO, 'FALTA_NAO_ABONADA', P_CDEMPRESA, P_CDFUNC, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, P_DTPONTO, '0000', ' 0000', 'S');
         
         V_AJUSTAPONTO := FALSE;
         
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20000, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;
   
   IF P_OPERACAO = 'INCLUIPONTO' THEN
      BEGIN
         NOVO_CODIGO('FPA_PONTO',V_ST_NRPONTO, 1, P_NRORG);
         V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
         
         V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(TO_CHAR(P_DTFIM, 'HH24MI'), TO_CHAR(P_DTINICIO, 'HH24MI'));
         V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRTEMPOCALCULADO, V_ALMOCOHORABASE);

         INSERT 
           INTO FPA_PONTO (NRORG, NRPONTO, IDPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRAUSENCIAALMOCO, HRREMUNERADA, HRSALDO, IDFALTA)
         VALUES (P_NRORG, V_NRPONTO, 'ATIVIDADE_INTERNA', P_CDEMPRESA, P_CDFUNC, P_DTPONTO, P_DTINICIO, P_DTINICIO, P_DTFIM, P_DTFIM, V_ALMOCOHORABASE, V_HRTEMPOCALCULADO, V_HRTEMPOCALCULADO, NVL(V_IDFALTA, 'N'));
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20000, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'ENTRADA' THEN
      BEGIN
         UPDATE FPA_PONTO
            SET DTENTRADA = P_DTINICIO,
                DSALTEENTRADA = P_DSOBSERVACAO
          WHERE NRPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20001, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'SAIDA' THEN
      BEGIN
         UPDATE FPA_PONTO
            SET DTSAIDA = P_DTFIM,
                DSALTESAIDA = P_DSOBSERVACAO 
          WHERE NRPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20002, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'AUTDEBBANCOHORAS' THEN
      BEGIN
         V_IDAUTDEBBH := SUBSTR(P_DSOBSERVACAO, 1, 1);
      
         UPDATE FPA_PONTO
            SET IDAUTDEBBANCOHORAS = V_IDAUTDEBBH
          WHERE NRPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20002, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'AUSENCIANAOABON' THEN
      BEGIN
         UPDATE FPA_PONTO
            SET HRAUSENCIANAOABON = P_HROPERACAO,
                DSALTEAUSENCIANAOABON = P_DSOBSERVACAO 
          WHERE NRPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20003, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'AUSENCIAABONADA' THEN
      BEGIN
         UPDATE FPA_PONTO
            SET HRAUSENCIAABONADA = P_HROPERACAO,
                DSALTEAUSENCIAABONADA = P_DSOBSERVACAO 
          WHERE NRPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20004, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'AUSENCIAEXTABONADA' THEN
      BEGIN
         UPDATE FPA_PONTO
            SET HRAUSENCIAEXTABON = P_HROPERACAO,
                DSALTEAUSENCIAEXTABON = P_DSOBSERVACAO 
          WHERE NRPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20005, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'AUSENCIA' OR P_OPERACAO = 'AUSENCIA_ALMOCO' THEN
      V_HRSAIDA := TO_CHAR(P_DTINICIO, 'HH24MI');
      V_HRENTRADA := TO_CHAR(P_DTFIM, 'HH24MI');
      V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRENTRADA, V_HRSAIDA);

      BEGIN
         NOVO_CODIGO_SC('FPA_AUSENCIAPONTO',V_ST_AUSENCIAPONTO, 1, P_NRORG);
         V_AUSENCIAPONTO := TO_NUMBER(V_ST_AUSENCIAPONTO);

         INSERT INTO FPA_AUSENCIAPONTO (NRAUSENCIAPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTINICIOAUSENCIA, DTFIMAUSENCIA, 
                                        HRAUSENCIA, IDAUSENCIAABONADA, IDAUSENCIANAOABON, IDAUSENCIAALMOCO, DSOBSERVACAOAUSENCIA)
              VALUES (V_AUSENCIAPONTO, P_CDEMPRESA, P_CDFUNC, P_DTPONTO, P_DTINICIO, P_DTFIM,
                      V_HRTEMPOCALCULADO, DECODE(P_OPERACAO, 'AUSENCIA', 'S', 'N'), 'N', DECODE(P_OPERACAO, 'AUSENCIA_ALMOCO', 'S', 'N'), P_DSOBSERVACAO);
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20006, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'ALMOCO' OR P_OPERACAO = 'AUSENCIA_ALMOCO' THEN
      BEGIN
         IF P_OPERACAO = 'AUSENCIA_ALMOCO' THEN
            V_NRAUSENCIAPONTO := V_AUSENCIAPONTO;
         ELSE
            V_NRAUSENCIAPONTO := P_NRPONTO;
         END IF;         
      
         SELECT DTINICIOAUSENCIA, DTFIMAUSENCIA, UTILIT.SUBT_HORA2(TO_CHAR(DTFIMAUSENCIA, 'HH24MI'), TO_CHAR(DTINICIOAUSENCIA, 'HH24MI'))
           INTO V_DTINICIOAUSENCIA, V_DTFIMAUSENCIA, V_ALMOCOAUSENCIA
           FROM FPA_AUSENCIAPONTO
          WHERE NRAUSENCIAPONTO = V_NRAUSENCIAPONTO;
			 
         SELECT DTENTRADA, DTSAIDA 
           INTO V_DTINICIOAUX, V_DTFIMAUX
			  FROM FPA_PONTO
			 WHERE NRORG = P_NRORG
				AND CDEMPRESA = P_CDEMPRESA
				AND CDFUNC = P_CDFUNC
				AND DTPONTO = P_DTPONTO;
			 
			 IF (NOT (V_DTINICIOAUSENCIA BETWEEN V_DTINICIOAUX AND V_DTFIMAUX)) OR
			    (NOT (V_DTFIMAUSENCIA BETWEEN V_DTINICIOAUX AND V_DTFIMAUX)) THEN 
					RAISE_APPLICATION_ERROR (-20000, 'ERROR_ALMOCO_FORA_PERIODO');    
			 END IF;
          
         IF (V_ALMOCOAUSENCIA < V_ALMOCOHORABASE) AND (V_ALMOCOHORABASE <> '0000') THEN

            V_DTFIMAUSENCIA := TO_DATE(TO_CHAR(V_DTINICIOAUSENCIA, 'DD/MM/YYYY')||' '||UTILIT.SOMA_HORA(TO_CHAR(V_DTINICIOAUSENCIA, 'HH24MI'), V_ALMOCOHORABASE), 'DD/MM/YYYY HH24MI');
         
            UPDATE FPA_AUSENCIAPONTO
               SET IDAUSENCIAALMOCO = 'S',
                   IDAUSENCIAABONADA = 'N',
                   IDAUSENCIANAOABON = 'N',
                   DTFIMAUSENCIA = V_DTFIMAUSENCIA 
             WHERE NRAUSENCIAPONTO = V_NRAUSENCIAPONTO;

            UPDATE FPA_PONTO
               SET HRAUSENCIAALMOCO = V_ALMOCOHORABASE
             WHERE NRORG = P_NRORG
               AND CDEMPRESA = P_CDEMPRESA
               AND CDFUNC = P_CDFUNC
               AND DTPONTO = P_DTPONTO;
               
         ELSIF (V_ALMOCOAUSENCIA > V_ALMOCOHORABASE) AND (V_ALMOCOHORABASE <> '0000') THEN
         
            V_DTFIMAUX := V_DTFIMAUSENCIA;
            V_DTFIMAUSENCIA := TO_DATE(TO_CHAR(V_DTINICIOAUSENCIA, 'DD/MM/YYYY')||' '||UTILIT.SOMA_HORA(TO_CHAR(V_DTINICIOAUSENCIA, 'HH24MI'), V_ALMOCOHORABASE), 'DD/MM/YYYY HH24MI');
            V_DTINICIOAUX := V_DTFIMAUSENCIA;
                     
            UPDATE FPA_AUSENCIAPONTO
               SET IDAUSENCIAALMOCO = 'S',
                   IDAUSENCIAABONADA = 'N',
                   IDAUSENCIANAOABON = 'N',
                   DTFIMAUSENCIA = V_DTFIMAUSENCIA 
             WHERE NRAUSENCIAPONTO = V_NRAUSENCIAPONTO;
             
            V_HRSAIDA := TO_CHAR(V_DTINICIOAUX, 'HH24MI');
            V_HRENTRADA := TO_CHAR(V_DTFIMAUX, 'HH24MI');
            V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRENTRADA, V_HRSAIDA);

            NOVO_CODIGO_SC('FPA_AUSENCIAPONTO',V_ST_AUSENCIAPONTO, 1, P_NRORG);
            V_AUSENCIAPONTO := TO_NUMBER(V_ST_AUSENCIAPONTO);

            INSERT INTO FPA_AUSENCIAPONTO (NRAUSENCIAPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTINICIOAUSENCIA, DTFIMAUSENCIA, 
                                           HRAUSENCIA, IDAUSENCIAABONADA, IDAUSENCIANAOABON, IDAUSENCIAALMOCO, DSOBSERVACAOAUSENCIA)
                 VALUES (V_AUSENCIAPONTO, P_CDEMPRESA, P_CDFUNC, P_DTPONTO, V_DTINICIOAUX, V_DTFIMAUX,
                         V_HRTEMPOCALCULADO, 'N', 'S', 'N', NULL);

            UPDATE FPA_PONTO
               SET HRAUSENCIAALMOCO = V_ALMOCOHORABASE
             WHERE NRORG = P_NRORG
               AND CDEMPRESA = P_CDEMPRESA
               AND CDFUNC = P_CDFUNC
               AND DTPONTO = P_DTPONTO;
               
         ELSE
         
            UPDATE FPA_AUSENCIAPONTO
               SET IDAUSENCIAALMOCO = 'S',
                   IDAUSENCIAABONADA = 'N',
                   IDAUSENCIANAOABON = 'N'
             WHERE NRAUSENCIAPONTO = V_NRAUSENCIAPONTO;
             
            UPDATE FPA_PONTO
               SET HRAUSENCIAALMOCO = V_ALMOCOAUSENCIA
             WHERE NRORG = P_NRORG
               AND CDEMPRESA = P_CDEMPRESA
               AND CDFUNC = P_CDFUNC
               AND DTPONTO = P_DTPONTO;
               
         END IF;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20007, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'ABONO' THEN
      BEGIN
         UPDATE FPA_AUSENCIAPONTO
            SET IDAUSENCIAALMOCO = 'N',
                IDAUSENCIAABONADA = 'S',
                IDAUSENCIANAOABON = 'N',
                DSOBSERVACAOAUSENCIA = P_DSOBSERVACAO
          WHERE NRAUSENCIAPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20008, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF P_OPERACAO = 'NAOABONO' THEN
      BEGIN
         UPDATE FPA_AUSENCIAPONTO
            SET IDAUSENCIAALMOCO = 'N',
                IDAUSENCIAABONADA = 'N',
                IDAUSENCIANAOABON = 'S',
                DSOBSERVACAOAUSENCIA = NULL
          WHERE NRAUSENCIAPONTO = P_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20009, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;       
   END IF;

   IF V_AJUSTAPONTO THEN
      BEGIN
         SELECT NRPONTO, DTENTRADA, DTSAIDA
           INTO V_NRPONTO, V_DTENTRADA, V_DTSAIDA      
           FROM FPA_PONTO
          WHERE NRORG = P_NRORG
            AND CDEMPRESA = P_CDEMPRESA
            AND CDFUNC = P_CDFUNC
            AND DTPONTO = P_DTPONTO;
      
         V_AUSENCIATOTAL := '0000';
         V_AUSENCIAABONADA := '0000';
         V_AUSENCIAEXTABON := '0000';
         V_AUSENCIANAOABON := '0000';
         V_AUSENCIAALMOCO := '0000';

         SELECT CASE WHEN MIN(DTINICIOAUSENCIA) < V_DTENTRADA THEN MIN(DTINICIOAUSENCIA) ELSE V_DTENTRADA END
           INTO V_DTHRENTRADA_ANT
           FROM FPA_AUSENCIAPONTO
          WHERE NRORG = P_NRORG
            AND CDEMPRESA = P_CDEMPRESA
            AND CDFUNC = P_CDFUNC
            AND DTPONTO = P_DTPONTO;
            
         OPEN C_FPAAUSENCIA;
         LOOP
            FETCH C_FPAAUSENCIA INTO V_NRAUSENCIAPONTO, V_DTHRSAIDA, V_DTHRENTRADA, V_HRTEMPOCALCULADO, V_IDTEMPOABONADO, V_IDTEMPONAOABON, V_IDALMOCO;
            EXIT WHEN C_FPAAUSENCIA%NOTFOUND;   

               IF TO_CHAR(V_DTHRSAIDA, 'DD/MM/YYYY HH24:MI') < TO_CHAR(V_DTHRENTRADA_ANT, 'DD/MM/YYYY HH24:MI') THEN
                  IF TO_CHAR(V_DTHRENTRADA, 'DD/MM/YYYY HH24:MI') <= TO_CHAR(V_DTHRENTRADA_ANT, 'DD/MM/YYYY HH24:MI') THEN
                     DELETE 
                       FROM FPA_AUSENCIAPONTO
                      WHERE NRAUSENCIAPONTO = V_NRAUSENCIAPONTO; 
                  ELSE
                     V_DTAUX := V_DTHRENTRADA_ANT+(1/24/60);  /* <-- Somando 1 minuto */
                  
                     V_HRSAIDA := TO_CHAR(V_DTAUX, 'HH24MI');
                     V_HRENTRADA := TO_CHAR(V_DTHRENTRADA, 'HH24MI');
                     
                     V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRENTRADA, V_HRSAIDA);

                     /* Os campos FPA_PONTO.HRAUSENCIATOTAL, FPA_PONTO.HRAUSENCIAABONADA, FPA_PONTO.HRAUSENCIANAOABON 
                        e FPA_PONTO.HRAUSENCIAALMOCO somente somaro os tempos ausentes constantes entre a Entrada e 
                        Sada da empresa (intrajornada) */
                     IF V_DTHRSAIDA < V_DTSAIDA AND V_DTAUX > V_DTENTRADA THEN 
                        IF V_IDTEMPOABONADO = 'S' THEN
                           V_AUSENCIAABONADA := UTILIT.SOMA_HORA(V_AUSENCIAABONADA, V_HRTEMPOCALCULADO);
                        END IF;   
                        IF V_IDTEMPONAOABON = 'S' THEN
                           V_AUSENCIANAOABON := UTILIT.SOMA_HORA(V_AUSENCIANAOABON, V_HRTEMPOCALCULADO);
                        END IF;   
                        IF V_IDALMOCO = 'S' THEN
                           V_AUSENCIAALMOCO := UTILIT.SOMA_HORA(V_AUSENCIAALMOCO, V_HRTEMPOCALCULADO);
                        END IF;   
                     ELSE
                        IF V_IDTEMPOABONADO = 'S' THEN
                           V_AUSENCIAEXTABON := UTILIT.SOMA_HORA(V_AUSENCIAEXTABON, V_HRTEMPOCALCULADO);
                        END IF;   
                     END IF;   
                     
                     UPDATE FPA_AUSENCIAPONTO
                        SET DTINICIOAUSENCIA = V_DTAUX,
                            HRAUSENCIA = V_HRTEMPOCALCULADO
                      WHERE NRAUSENCIAPONTO = V_NRAUSENCIAPONTO; 
                      
                     V_DTHRENTRADA_ANT := V_DTHRENTRADA;
                  END IF;
               ELSE
                  V_HRSAIDA := TO_CHAR(V_DTHRSAIDA, 'HH24MI');
                  V_HRENTRADA := TO_CHAR(V_DTHRENTRADA, 'HH24MI');
                  
                  V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRENTRADA, V_HRSAIDA);
                  
                  /* Os campos FPA_PONTO.HRAUSENCIATOTAL, FPA_PONTO.HRAUSENCIAABONADA, FPA_PONTO.HRAUSENCIANAOABON 
                     e FPA_PONTO.HRAUSENCIAALMOCO somente somaro os tempos ausentes constantes entre a Entrada e 
                     Sada da empresa (intrajornada) */
                  IF V_DTHRSAIDA < V_DTSAIDA AND V_DTHRENTRADA > V_DTENTRADA THEN 
                     IF V_IDTEMPOABONADO = 'S' THEN
                        V_AUSENCIAABONADA := UTILIT.SOMA_HORA(V_AUSENCIAABONADA, V_HRTEMPOCALCULADO);
                     END IF;   
                     IF V_IDTEMPONAOABON = 'S' THEN
                        V_AUSENCIANAOABON := UTILIT.SOMA_HORA(V_AUSENCIANAOABON, V_HRTEMPOCALCULADO);
                     END IF;   
                     IF V_IDALMOCO = 'S' THEN
                        V_AUSENCIAALMOCO := UTILIT.SOMA_HORA(V_AUSENCIAALMOCO, V_HRTEMPOCALCULADO);
                     END IF;   
                  ELSE
                     IF V_IDTEMPOABONADO = 'S' THEN
                        V_AUSENCIAEXTABON := UTILIT.SOMA_HORA(V_AUSENCIAEXTABON, V_HRTEMPOCALCULADO);
                     END IF;   
                  END IF;   
                  
                  UPDATE FPA_AUSENCIAPONTO
                     SET HRAUSENCIA = V_HRTEMPOCALCULADO
                   WHERE NRAUSENCIAPONTO = V_NRAUSENCIAPONTO;
                   
                  V_DTHRENTRADA_ANT := V_DTHRENTRADA;
               END IF;
         END LOOP;
         CLOSE C_FPAAUSENCIA;
         
         /* Tratamento para ESTAGIRIOS */ 
         IF V_CDTPADMISSAO IN ('B', 'E', 'O') THEN
            IF V_AUSENCIANAOABON > '0010' THEN
               V_AUSENCIAABONADA := UTILIT.SOMA_HORA(V_AUSENCIAABONADA, '0010');
               V_AUSENCIANAOABON := UTILIT.SUBT_HORA2(V_AUSENCIANAOABON, '0010');
            ELSE
               V_AUSENCIAABONADA := UTILIT.SOMA_HORA(V_AUSENCIAABONADA, V_AUSENCIANAOABON);
               V_AUSENCIANAOABON := '0000';
            END IF;
         END IF;

         V_AUSENCIAABONADAORIG := V_AUSENCIAABONADA;
         V_AUSENCIAEXTABONORIG := V_AUSENCIAEXTABON;
         V_AUSENCIANAOABONORIG := V_AUSENCIANAOABON;
         
         SELECT DECODE(DSALTEAUSENCIAABONADA, NULL, V_AUSENCIAABONADA, HRAUSENCIAABONADA),
                DECODE(DSALTEAUSENCIAEXTABON, NULL, V_AUSENCIAEXTABON, HRAUSENCIAEXTABON), 
                DECODE(DSALTEAUSENCIANAOABON, NULL, V_AUSENCIANAOABON, HRAUSENCIANAOABON) 
           INTO V_AUSENCIAABONADA,
                V_AUSENCIAEXTABON, 
                V_AUSENCIANAOABON
           FROM FPA_PONTO
          WHERE NRORG = P_NRORG
            AND CDEMPRESA = P_CDEMPRESA
            AND CDFUNC = P_CDFUNC
            AND DTPONTO = P_DTPONTO;

         V_HRENTRADA := TO_CHAR(V_DTENTRADA, 'HH24MI');
         V_HRSAIDA := TO_CHAR(V_DTSAIDA, 'HH24MI');
         V_HRTEMPOCALCULADO := UTILIT.SUBT_HORA2(V_HRSAIDA, V_HRENTRADA);
         V_HORASREMUNERADAS := UTILIT.SUBT_HORA2(V_HRTEMPOCALCULADO, V_AUSENCIAALMOCO);
         V_HORASREMUNERADAS := UTILIT.SUBT_HORA2(V_HORASREMUNERADAS, V_AUSENCIANAOABON);
         V_HORASREMUNERADAS := UTILIT.SOMA_HORA(V_HORASREMUNERADAS, V_AUSENCIAEXTABON);
         
         IF V_HORASREMUNERADAS > V_QTHORASBASE THEN
            IF (V_CDTPADMISSAO IN ('S', 'P')) THEN
               V_DEBITOCREDITOHORAS := '0000';
            ELSE
               V_DEBITOCREDITOHORAS := UTILIT.SUBT_HORA2(V_HORASREMUNERADAS, V_QTHORASBASE);
            END IF;            
            
            /* Tratamento para Scios e PJ (V_QTHORASBASE <> '0000') */

            /* #### Clculo atual considerando banco de horas #### */
            V_DEBITOCREDITOHORASEXTRA := ' 0000';
            
            /* #### Clculo considerando horas extras a partir de 02:00 horas positivas (funcionrios alocados em MG) 
				 * ou todo saldo positivo como hora extra (funcionrios alocados em outros estados) 
				 * ou todo saldo positivo como hora extra (Sb, Dom e Feriados - Solicitado por Gabrielle em 13/11/2014) #### */
            IF NOT (V_CDTPADMISSAO IN ('B', 'E', 'O')) THEN                
               IF V_SGESTADO_FUNC = 'MG' THEN
                  IF (NOT (V_CDTPADMISSAO IN ('S', 'P'))) THEN
							IF V_SAB_DOM_FERIADO = 'N' THEN 
								IF (V_DEBITOCREDITOHORAS > '0200') THEN
									V_DEBITOCREDITOHORASEXTRA := UTILIT.SUBT_HORA2(V_DEBITOCREDITOHORAS, '0200');
								END IF;               
							ELSE
								V_DEBITOCREDITOHORASEXTRA := V_DEBITOCREDITOHORAS;
							END IF;	
                  END IF;   
               ELSE
                  V_DEBITOCREDITOHORASEXTRA := V_DEBITOCREDITOHORAS;
               END IF;               
            END IF;   
         ELSE
            V_DEBITOCREDITOHORASEXTRA := ' 0000';
            IF V_HORASREMUNERADAS < V_QTHORASBASE THEN
               V_DEBITOCREDITOHORAS := '-'||UTILIT.SUBT_HORA2(V_QTHORASBASE, V_HORASREMUNERADAS);
            ELSE
               V_DEBITOCREDITOHORAS := ' '||UTILIT.SUBT_HORA2(V_QTHORASBASE, V_HORASREMUNERADAS);
            END IF;   
         END IF;   

         V_AUSENCIATOTAL := UTILIT.SOMA_HORA(V_AUSENCIAABONADA, V_AUSENCIANAOABON);

         UPDATE FPA_PONTO
            SET DTENTRADA = V_DTENTRADA,
                DTSAIDA = V_DTSAIDA,
                HRAUSENCIATOTAL = V_AUSENCIATOTAL,
                HRAUSENCIAABONADAORIG = V_AUSENCIAABONADAORIG,
                HRAUSENCIAABONADA = V_AUSENCIAABONADA,
                HRAUSENCIAEXTABONORIG = V_AUSENCIAEXTABONORIG,
                HRAUSENCIAEXTABON = V_AUSENCIAEXTABON,
                HRAUSENCIANAOABONORIG = V_AUSENCIANAOABONORIG,
                HRAUSENCIANAOABON = V_AUSENCIANAOABON,
                HRREMUNERADA = V_HORASREMUNERADAS,
                HRSALDO = V_DEBITOCREDITOHORAS,
                HRSALDOHORAEXTRA = V_DEBITOCREDITOHORASEXTRA
          WHERE NRPONTO = V_NRPONTO;
      EXCEPTION
         WHEN OTHERS THEN BEGIN
            V_ERROR_CODE := SQLCODE;
            V_ERROR_MESSAGE := SQLERRM;
            RAISE_APPLICATION_ERROR (-20010, 'ERROR NUMBER: '||TO_CHAR(SQLCODE)||' MESSAGE: '||V_ERROR_MESSAGE);
         END;
      END;   
      
   END IF;
   
/* ##################################################################################################################### */
/* ################################## TOTALIZAO DO PONTO NO PERODO (INCIO) ######################################### */
/* ##################################################################################################################### */

   BEGIN
      SELECT NVL(IDACERTOBANCOHORAS, 'N')
        INTO V_IDACERTOBH_ANT
        FROM FPA_TOTALIZAPONTO
		 WHERE NRORG = P_NRORG
			AND CDEMPRESA = P_CDEMPRESA
			AND CDFUNC = P_CDFUNC
			AND NRPERIODOAPURAPONTO = V_NRPERIODOAPURAPONTO;
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_IDACERTOBH_ANT := 'N';
      END;
   END;
	
   DELETE 
     FROM FPA_TOTALIZAPONTO
    WHERE NRORG = P_NRORG
      AND CDEMPRESA = P_CDEMPRESA
      AND CDFUNC = P_CDFUNC
      AND NRPERIODOAPURAPONTO = V_NRPERIODOAPURAPONTO;

   /* Para tratamento do desconto do DSR em caso de falta no abonada, o perodo considerado 
      sempre iniciar numa segunda-feira e terminar num domingo */ 
   IF TO_CHAR(V_DTINICIALAPURAPONTO, 'D') = '1'  THEN /* 1 -> Domingo */ 
      V_DTINICIOAUX := TRUNC(V_DTINICIALAPURAPONTO-6);
   ELSE
      V_DTINICIOAUX := TRUNC(V_DTINICIALAPURAPONTO+(2-TO_NUMBER(TO_CHAR(V_DTINICIALAPURAPONTO, 'D'))));
   END IF;   
   V_DTFIMAUX := TRUNC(V_DTFINALAPURAPONTO+(8-TO_NUMBER(TO_CHAR(V_DTFINALAPURAPONTO, 'D'))));
   
   /* Verificao saldo de banco de horas do perodo de apurao anterior */
   P_DTPERIODO_ANTE := V_DTINICIALAPURAPONTO - 1;
   
   SELECT P.NRPERIODOAPURAPONTO
     INTO V_NRPERIODOAPURA_ANTE
     FROM FPA_PERIODOAPURAPONTO P, FUNCIONARIO F
    WHERE F.CDEMPRESA = P_CDEMPRESA
      AND F.CDFUNC = P_CDFUNC
      AND F.NRORG = P_NRORG
      AND P.CDTPADMISSAO = V_CDTPADMISSAO
      AND P.DTINICIALAPURAPONTO <= P_DTPERIODO_ANTE
      AND P.DTFINALAPURAPONTO >= P_DTPERIODO_ANTE;
   
   BEGIN
      SELECT HRSALDOBANCOHORAS, HRSALDOBHRSACRESC, NVL(IDACERTOBANCOHORAS, 'N')
        INTO V_HRSALDOBANCOHORAS, V_HRSALDOBHRSACRESC, V_IDACERTOBANCOHORAS
        FROM FPA_TOTALIZAPONTO
       WHERE NRORG = P_NRORG
         AND CDEMPRESA = P_CDEMPRESA
         AND CDFUNC = P_CDFUNC
         AND NRPERIODOAPURAPONTO = V_NRPERIODOAPURA_ANTE;
         
      IF (V_IDACERTOBANCOHORAS = 'S') OR (V_HRSALDOBANCOHORAS IS NULL) THEN
         V_HRSALDOBANCOHORAS := ' 0000';
      END IF;    

      IF (V_IDACERTOBANCOHORAS = 'S') OR (V_HRSALDOBHRSACRESC IS NULL) THEN
         V_HRSALDOBHRSACRESC := ' 0000';
         V_HRSALDOBANCOHORAS := ' 0000';
      END IF;    
   EXCEPTION
      WHEN NO_DATA_FOUND THEN BEGIN
         V_HRSALDOBANCOHORAS := ' 0000';
         V_HRSALDOBHRSACRESC := ' 0000';
      END;
   END;

   V_HRSALDOBH_MES := '0000';
   V_TOT_HRSALDO := '0000';
   V_TOT_HRREMUNERADA := '0000';
   V_HREXTRA25BANCOHORAS := '0000';
   V_HREXTRA50BANCOHORAS := '0000';
   V_HREXTRA100BANCOHORAS := '0000';
   V_HRCOMPBANCOHORAS := '0000';
   V_HRSALDOHORASFALTA := '0000';
   V_HREXTRA100 := '0000';

   V_QTDIASDESCONTOFALTA := 0;
   V_QTDIASDESCONTODSR := 0;
   V_FALTA_NA_SEMANA := FALSE;

   OPEN C_FPA_PONTO_TOTAL;
   LOOP
      FETCH C_FPA_PONTO_TOTAL INTO V_DTPONTO, V_SAB_DOM_FERIADO, V_DIASEMPTO, V_NRPONTO, V_IDPONTO, V_DEBITOCREDITOHORAS, V_DEBITOCREDITOHORASEXTRA, 
                                   V_ALMOCOAUSENCIA, V_HORASJORNADA, V_HRREMUNERADA, V_IDFALTA, V_QT_ALMOCO_HB, V_IDAUTDEBBH;
      EXIT WHEN C_FPA_PONTO_TOTAL%NOTFOUND;
      
      V_ALMOCOHB_TOT := SUBSTR(V_QT_ALMOCO_HB,1,4);
      V_QTHB_TOT := SUBSTR(V_QT_ALMOCO_HB,6,4);
                                                                                         /* Nos casos de horrio base inferiores a 06:00, o almoo no  obrigatrio.  */  
                                                                                         /* Quando existe horrio base, a totalizao de horas do dia ser considerada */  
                                                                                         /* apenas aps o usurio ter informado o intervalo de almoo.                 */  
      IF (V_DTPONTO >= V_DTINICIALAPURAPONTO) AND (V_DTPONTO <= V_DTFINALAPURAPONTO) AND (V_IDPONTO = 'ATIVIDADE_EXTERNA' OR 
                                                                                          V_QTHB_TOT <= '0600' OR 
                                                                                          V_HORASJORNADA <= '0600' OR
                                                                                         ((V_ALMOCOHB_TOT <> '0000') AND (V_ALMOCOAUSENCIA <> '0000'))) THEN 
         
         /* Clculo do total de horas remuneradas e do total do saldo de horas alm jornada */
         V_TOT_HRREMUNERADA := UTILIT.SOMA_HORA2(V_TOT_HRREMUNERADA, V_HRREMUNERADA);
         IF NOT (V_CDTPADMISSAO IN ('S', 'P')) THEN
            V_TOT_HRSALDO := UTILIT.SOMA_HORA2(V_TOT_HRSALDO, V_DEBITOCREDITOHORAS);
         ELSE 
            V_DEBITOCREDITOHORAS := '0000';
         END IF;   
         
         /* Totalizao de horas extras */
         IF NOT (V_CDTPADMISSAO IN ('B', 'E', 'O')) THEN                
            V_HREXTRA100 := UTILIT.SOMA_HORA2(V_HREXTRA100, V_DEBITOCREDITOHORASEXTRA);
         END IF;   
         
         /* Clculos para banco de horas (total, 25%, 50%, 100% e hrs compensadas) */
         V_HREXTRA25BH := '0000';
         V_HREXTRA25BH_ACRESC := '0000';
         V_HREXTRA50BH := '0000';
         V_HREXTRA50BH_ACRESC := '0000';
         V_HREXTRA100BH := '0000';
         V_HREXTRA100BH_ACRESC := '0000';
         V_HRCOMPBH := '0000';
         
         V_HRFALTA := '0000';
         
         /* O saldo dirio para banco de horas ser, no mximo, 02:00 em dias teis. Alm disso ser considerado hora extra. 
          * Em Sb/Dom e feriados todas as horas iro para o banco de horas. Tratamento realizado apenas para MG (at 12/11/2014)
          * Em Sb/Dom e feriados todas as horas sero extras. Solicitado por Gabrielle (aps 13/11/2014) */
         IF (TO_NUMBER(V_DEBITOCREDITOHORAS) = 0) OR (SUBSTR(V_DEBITOCREDITOHORAS,1,1) = '-') THEN
            V_DEBCREDBANCOHORAS := V_DEBITOCREDITOHORAS;
         ELSE
				IF (V_SAB_DOM_FERIADO = 'N') THEN
					IF (TO_NUMBER(V_DEBITOCREDITOHORASEXTRA) > 0) AND (V_DEBITOCREDITOHORAS > '0200') AND (V_SGESTADO_FUNC = 'MG') THEN
						V_DEBCREDBANCOHORAS := '0200';
					ELSE
						V_DEBCREDBANCOHORAS := V_DEBITOCREDITOHORAS;
					END IF;   
				ELSE
					V_DEBCREDBANCOHORAS := '0000';
				END IF;
         END IF;   
            
         /* Tratamento temporrio. O primeiro perodo aps alterao no clculo do banco 
          * de horas - 21/08 a 20/09/2013 - debitar todos os saldos negativos no B.H. */
         IF (SUBSTR(V_DEBCREDBANCOHORAS,1,1) = '-') AND (V_CDTPADMISSAO = 'C') AND 
            (V_DTPONTO BETWEEN TO_DATE('21/08/2013', 'DD/MM/YYYY') AND TO_DATE('20/09/2013', 'DD/MM/YYYY')) AND
            (NOT (P_CDFUNC IN ('0000002187', '0000001808', '0000001069', '0000002817', '0000002801', '0000002321', '0000002841', '0000002535', '0000002499', '0000002353', '0000002700', '0000001238', '0000001872', '0000002812', '0000002750'))) THEN
            V_IDAUTDEBBH := 'S';
         END IF;
         IF (SUBSTR(V_DEBCREDBANCOHORAS,1,1) = '-') AND (V_CDTPADMISSAO = 'C') AND 
            (V_DTPONTO BETWEEN TO_DATE('21/08/2013', 'DD/MM/YYYY') AND TO_DATE('20/09/2013', 'DD/MM/YYYY')) AND
            (P_CDFUNC IN ('0000002187', '0000001808', '0000001069', '0000002817', '0000002801', '0000002321', '0000002841', '0000002535', '0000002499', '0000002353', '0000002700', '0000001238', '0000001872', '0000002812', '0000002750')) THEN
            V_IDAUTDEBBH := 'N';
         END IF;
         
         IF NOT (V_CDTPADMISSAO IN ('S', 'P', 'E')) THEN
            IF SUBSTR(V_DEBCREDBANCOHORAS,1,1) = '-' THEN
               IF (V_IDAUTDEBBH = 'S') AND (V_SGESTADO_FUNC = 'MG') THEN
                  V_HRCOMPBH := V_DEBCREDBANCOHORAS;
               ELSE
                  IF NOT (V_CDTPADMISSAO IN ('B', 'E', 'O')) THEN
                     V_HRFALTA := V_DEBCREDBANCOHORAS;
                  END IF;   
               END IF;               
            ELSE
               V_IDAUTDEBBH := 'N'; /* Com saldo positivo de horas, no h autorizao para dbito em banco de horas  */
               IF (NOT (V_CDTPADMISSAO IN ('B', 'E', 'O'))) AND (V_SGESTADO_FUNC = 'MG') THEN                
                  V_HRSALDOBH_MES := UTILIT.SOMA_HORA2(V_HRSALDOBH_MES, V_DEBCREDBANCOHORAS);
                  
                  IF V_SAB_DOM_FERIADO = 'N' THEN
					  /* Sb, Dom e Feriados sero considerados como horas extras
						* conforme solicitado por Gabrielle em 13/11/2014
						*
                  IF V_SAB_DOM_FERIADO = 'S' THEN
  						   V_HREXTRA100BH := V_DEBCREDBANCOHORAS;
                     V_HREXTRA100BH_ACRESC := UTILIT.MULT_HORA(V_DEBCREDBANCOHORAS, 2);
                  ELSE
						*/
                     IF V_HRSALDOBH_MES > '3000' THEN
                        V_HRAUX := UTILIT.SOMA_HORA2(V_HRSALDOBH_MES, '-3000');
                        IF V_HRAUX < SUBSTR(V_DEBCREDBANCOHORAS, -4) THEN
                           V_HREXTRA25BH := UTILIT.SOMA_HORA2(V_DEBCREDBANCOHORAS, '-'||SUBSTR(V_HRAUX, -4));
                           V_HREXTRA50BH := V_HRAUX;

                           V_HREXTRA25BH_ACRESC := UTILIT.MULT_HORA(UTILIT.SOMA_HORA2(V_DEBCREDBANCOHORAS, '-'||SUBSTR(V_HRAUX, -4)), 1.25);
                           V_HREXTRA50BH_ACRESC := UTILIT.MULT_HORA(V_HRAUX, 1.5);
                        ELSE
                           V_HREXTRA50BH := V_DEBCREDBANCOHORAS;
                           V_HREXTRA50BH_ACRESC := UTILIT.MULT_HORA(V_DEBCREDBANCOHORAS, 1.5);
                        END IF;   
                     ELSE
                        V_HREXTRA25BH := V_DEBCREDBANCOHORAS;
                        V_HREXTRA25BH_ACRESC := UTILIT.MULT_HORA(V_DEBCREDBANCOHORAS, 1.25);
                     END IF;
                  END IF;   
               END IF;   
            END IF;   
         END IF;   
         
         V_HRSALDOBANCOHORAS := UTILIT.SOMA_HORA2(V_HRSALDOBANCOHORAS, V_HRCOMPBH);
         V_HRSALDOBANCOHORAS := UTILIT.SOMA_HORA2(V_HRSALDOBANCOHORAS, V_HREXTRA25BH);
         V_HRSALDOBANCOHORAS := UTILIT.SOMA_HORA2(V_HRSALDOBANCOHORAS, V_HREXTRA50BH);
         V_HRSALDOBANCOHORAS := UTILIT.SOMA_HORA2(V_HRSALDOBANCOHORAS, V_HREXTRA100BH);
         
         V_HRSALDOBHRSACRESC := UTILIT.SOMA_HORA2(V_HRSALDOBHRSACRESC, V_HRCOMPBH);
         V_HRSALDOBHRSACRESC := UTILIT.SOMA_HORA2(V_HRSALDOBHRSACRESC, V_HREXTRA25BH_ACRESC);
         V_HRSALDOBHRSACRESC := UTILIT.SOMA_HORA2(V_HRSALDOBHRSACRESC, V_HREXTRA50BH_ACRESC);
         V_HRSALDOBHRSACRESC := UTILIT.SOMA_HORA2(V_HRSALDOBHRSACRESC, V_HREXTRA100BH_ACRESC);
         
         /* Tratamento para bloqueio da possibilidade de compensao de horas quando
		    no houver saldo suficiente para essa finalidade. */
		 IF (TO_NUMBER(V_HRCOMPBH) < 0) AND (TO_NUMBER(V_HRSALDOBHRSACRESC) < 0) THEN 
            RAISE_APPLICATION_ERROR (-20000, 'ERROR_COMPENSAR_HORAS_'||TO_CHAR(V_DTPONTO, 'DD/MM/YYYY'));    
 		 END IF;
		 
		 IF NOT (V_NRPONTO IS NULL) THEN
            UPDATE FPA_PONTO
            SET HRSALDO = V_DEBITOCREDITOHORAS,
                HRSALDO25 = SUBSTR(V_HREXTRA25BH, -4),
                HRSALDO50 = SUBSTR(V_HREXTRA50BH, -4),
                HRSALDO100 = SUBSTR(V_HREXTRA100BH, -4),
                HRSALDOCOMP = V_HRCOMPBH,
                HRSALDOBANCOHORAS = V_HRSALDOBANCOHORAS,
                HRSALDOBHRSACRESC = V_HRSALDOBHRSACRESC,
                HRSALDOHORASFALTA = V_HRFALTA,
                IDAUTDEBBANCOHORAS = V_IDAUTDEBBH
             WHERE NRPONTO = V_NRPONTO;
         END IF;
         
         V_HREXTRA25BANCOHORAS := UTILIT.SOMA_HORA2(V_HREXTRA25BANCOHORAS, V_HREXTRA25BH);
         V_HREXTRA50BANCOHORAS := UTILIT.SOMA_HORA2(V_HREXTRA50BANCOHORAS, V_HREXTRA50BH);
         V_HREXTRA100BANCOHORAS := UTILIT.SOMA_HORA2(V_HREXTRA100BANCOHORAS, V_HREXTRA100BH);
         V_HRCOMPBANCOHORAS := UTILIT.SOMA_HORA2(V_HRCOMPBANCOHORAS, V_HRCOMPBH);
         V_HRSALDOHORASFALTA := UTILIT.SOMA_HORA2(V_HRSALDOHORASFALTA, V_HRFALTA);
   
         /* Tratamento para controle de faltas e DSR */
         IF (V_IDFALTA = 'S') AND (V_DIASEMPTO <> '1') THEN /* 1 -> Domingo */
            V_QTDIASDESCONTOFALTA := V_QTDIASDESCONTOFALTA + 1;
         END IF;   
         
         IF (V_DIASEMPTO = '1') AND V_FALTA_NA_SEMANA THEN 
            V_QTDIASDESCONTODSR := V_QTDIASDESCONTODSR + 1;
         END IF;
      END IF;
      
      /* Tratamento para controle de faltas e DSR */
      IF (V_IDFALTA = 'S') AND (V_DIASEMPTO <> '1') THEN /* 1 -> Domingo */
         V_FALTA_NA_SEMANA := TRUE;
      END IF;   
      
      IF (V_DIASEMPTO = '1') THEN 
         IF V_FALTA_NA_SEMANA THEN         
            V_FALTA_NA_SEMANA := FALSE;
            
            IF V_NRPONTO IS NULL THEN
               NOVO_CODIGO('FPA_PONTO', V_ST_NRPONTO, 1, P_NRORG);
            
               INSERT 
                 INTO FPA_PONTO (NRORG, NRPONTO, IDPONTO, CDEMPRESA, CDFUNC, DTPONTO, DTENTRADAORIG, DTENTRADA, DTSAIDAORIG, DTSAIDA, HRREMUNERADA, HRSALDO, IDFALTA)
               VALUES (P_NRORG, TO_NUMBER(V_ST_NRPONTO), 'FALTA_NAO_ABONADA', P_CDEMPRESA, P_CDFUNC, V_DTPONTO, V_DTPONTO, V_DTPONTO, V_DTPONTO, V_DTPONTO, '0000', ' 0000', 'S');
            ELSE
               UPDATE FPA_PONTO
                  SET IDFALTA = 'S'
                WHERE NRPONTO = V_NRPONTO;
            END IF;
         ELSIF (V_NRPONTO IS NOT NULL) THEN
            IF (V_IDPONTO = 'FALTA_NAO_ABONADA') THEN
               DELETE FROM FPA_PONTO
                WHERE NRPONTO = V_NRPONTO;
            ELSE
               UPDATE FPA_PONTO
                  SET IDFALTA = 'N'
                WHERE NRPONTO = V_NRPONTO;
            END IF;
         END IF;            
      END IF;

   END LOOP;
   CLOSE C_FPA_PONTO_TOTAL;

   /* Gravao da linha de totalizao do ponto no perodo */
   NOVO_CODIGO('FPA_TOTALIZAPONTO', V_ST_NRPONTO, 1, P_NRORG);
   V_NRPONTO := TO_NUMBER(V_ST_NRPONTO);
   
   INSERT INTO FPA_TOTALIZAPONTO (NRTOTALIZAPONTO, NRORG, CDEMPRESA, CDFUNC, CDEMPRESAFOLHA, NRPERIODOAPURAPONTO, 
                                  HREXTRA25BANCOHORAS, HREXTRA50BANCOHORAS, HREXTRA100BANCOHORAS, HRCOMPBANCOHORAS, HRSALDOBANCOHORAS, HRSALDOBHRSACRESC,
                                  HREXTRA100, HRSALDOHORASFALTA, HRREMUNERADA, HRSALDO, QTDIASDESCONTOFALTA, QTDIASDESCONTODSR, IDACERTOBANCOHORAS)
        VALUES (V_NRPONTO, P_NRORG, P_CDEMPRESA, P_CDFUNC, V_CDEMPRESAFOLHA, V_NRPERIODOAPURAPONTO, 
                V_HREXTRA25BANCOHORAS, V_HREXTRA50BANCOHORAS, V_HREXTRA100BANCOHORAS, V_HRCOMPBANCOHORAS, V_HRSALDOBANCOHORAS, V_HRSALDOBHRSACRESC,
                V_HREXTRA100, V_HRSALDOHORASFALTA, V_TOT_HRREMUNERADA, V_TOT_HRSALDO, V_QTDIASDESCONTOFALTA, V_QTDIASDESCONTODSR, V_IDACERTOBH_ANT);         
   
/* ##################################################################################################################### */
/* ################################## TOTALIZAO DO PONTO NO PERODO (TRMINO) ######################################## */
/* ##################################################################################################################### */

   COMMIT;
   
   IF V_SEM_BATIDA_PONTO THEN
      RAISE_APPLICATION_ERROR (-20000, 'ERROR_SEM_BATIDAPONTO');    
   END IF;
END;
/